import { CommonModule } from '@angular/common';
import { Directive, ElementRef, Input, TemplateRef, Component, ContentChildren, EventEmitter, HostBinding, HostListener, NgZone, Output, ViewChild, ViewEncapsulation, forwardRef, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
/**
 * The ToggleLabel directive allows you to customize the label for the "On" and "Off" states,
 * allowing for more robust and complex displays.
 * This directive must be used in conjunction with a ng-template.
 */
class NgToggleLabel {
    /**
     * @param {?} templateRef
     * @param {?} elRef
     */
    constructor(templateRef, elRef) {
        this.templateRef = templateRef;
        this.elRef = elRef;
    }
    /**
     * @return {?}
     */
    get element() {
        return this.elRef;
    }
}
NgToggleLabel.decorators = [
    { type: Directive, args: [{ selector: 'ng-template[ngToggleLabel]' },] }
];
/** @nocollapse */
NgToggleLabel.ctorParameters = () => [
    { type: TemplateRef },
    { type: ElementRef }
];
NgToggleLabel.propDecorators = {
    forLabel: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
/**
 * The Toggle directive allows for standalone or checkbox-enabled switch toggling via a UI element.
 * The toggle is styled using Bootstrap v4+ classes.
 */
class NgToggle {
    /**
     * @param {?} ngZone
     * @param {?} element
     */
    constructor(ngZone, element) {
        this.ngZone = ngZone;
        this.element = element;
        /**
         * Display text when toggled in the "On" position
         */
        this.onText = 'On';
        /**
         * Display text when toggled in the "Off" position
         */
        this.offText = 'Off';
        /**
         * Bootstrap color scheme when toggled in the "On" position (i.e. "primary" translates to "btn-primary")
         */
        this.onColor = 'primary';
        /**
         * Bootstrap color scheme when toggled in the "Off" position (i.e. "primary" translates to "btn-primary")
         */
        this.offColor = 'secondary';
        /**
         * Button size to display the toggle
         */
        this.size = '';
        /**
         * Whether the toggle is disabled or not
         */
        this.disabled = false;
        /**
         * An event fired when the user causes a change.
         * The payload of the event is the currently selected value.
         */
        this.valueChange = new EventEmitter();
        this.btnClass = true;
        this.width = 0;
        this.handleWidth = 0;
        this._animate = true;
        this._innerAnimate = true;
        this._innerState = false;
        this._innerWidth = 'auto';
        this._disableInitialAnimation = false;
        this._dragStart = null;
        this._dragEnd = null;
        this._initialized = false;
        this._hidden = false;
    }
    /**
     * Optional. Enable/Disable the initial transition animation. May prevent unwanted animation display.
     * @param {?} value
     * @return {?}
     */
    set outerAnimate(value) {
        this._disableInitialAnimation = value;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set value(value) {
        this.setState(value);
    }
    /**
     * @return {?}
     */
    get value() {
        return this._innerState;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.calculateWidth();
        this._initialized = true;
        this.ngZone.run(() => setTimeout(() => this._disableInitialAnimation = false));
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        /** @type {?} */
        const hidden = this.element.nativeElement.offsetParent === null;
        if ((!this._initialized || this.width === 0) && this._hidden !== hidden) {
            this._initialized = false;
            this.calculateWidth();
            this._initialized = true;
        }
        this._hidden = hidden;
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        /** @type {?} */
        const onElement = this.labelElements.find((item) => item.forLabel.toLowerCase() === 'on');
        /** @type {?} */
        const offElement = this.labelElements.find((item) => item.forLabel.toLowerCase() === 'off');
        if (onElement) {
            this.onLabel = onElement;
        }
        if (offElement) {
            this.offLabel = offElement;
        }
        if (onElement || offElement) {
            this.calculateWidth(true);
        }
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if ('onText' in changes || 'offText' in changes || 'size' in changes) {
            this._initialized = false;
            this.calculateWidth(this._initialized);
            this._initialized = true;
        }
    }
    /**
     * @return {?}
     */
    get largeButton() {
        return this.size === 'lg';
    }
    /**
     * @return {?}
     */
    get smallButton() {
        return this.size === 'sm';
    }
    /**
     * @return {?}
     */
    get handleDark() {
        return (this.value && this.onColor === 'light') || (!this.value && this.offColor === 'light');
    }
    /**
     * @return {?}
     */
    get indeterminate() {
        return this._innerState === null || typeof this._innerState === 'undefined';
    }
    /**
     * @return {?}
     */
    get toggledOn() {
        return this.innerState === true;
    }
    /**
     * @return {?}
     */
    get toggledOff() {
        return this.innerState === false;
    }
    /**
     * @return {?}
     */
    get innerState() {
        return this._innerState;
    }
    /**
     * @return {?}
     */
    get animate() {
        return this._animate && (!this._disableInitialAnimation || this._initialized && !this._disableInitialAnimation);
    }
    /**
     * @return {?}
     */
    get marginLeft() {
        /** @type {?} */
        let margin = 0;
        if (!this._initialized) ;
        else if (this.indeterminate || this._innerState === null || typeof this._innerState === 'undefined') {
            margin = -(this.width / 2);
        }
        else if (this._dragEnd) {
            margin = this._dragEnd;
        }
        else if (!this._innerState) {
            margin = -this.width;
        }
        return margin + 'px';
    }
    /**
     * @return {?}
     */
    handleResize() {
        this._initialized = false;
        this.calculateWidth(true);
        this._initialized = true;
    }
    /**
     * @return {?}
     */
    handleClick() {
        if (!this.disabled && !this._dragEnd) {
            this.setState(!this._innerState);
        }
        else if (this._dragEnd) {
            this._dragEnd = null;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTouchStart(event) {
        this.onDragStart(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseDown(event) {
        this.onDragStart(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTouchMove(event) {
        this.onDragMove(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseMove(event) {
        this.onDragMove(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onTouchEnd(event) {
        this.onDragEnd(event, true);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseUp(event) {
        this.onDragEnd(event);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onMouseLeave(event) {
        this.onDragEnd(event, true);
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onKeyDown(event) {
        if (!event.key || this.disabled) {
            return;
        }
        switch (event.key) {
            case 'Left':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(false);
                break;
            case 'Right':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(true);
                break;
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragStart(event) {
        if (event.target === this.handle$) {
            if (this._dragStart || this.disabled) {
                return;
            }
            event.preventDefault();
            event.stopPropagation();
            this._dragStart = (event.pageX || event.touches[0].pageX) - parseInt(this.container$.style.marginLeft, 10);
            if (this._animate) {
                this._animate = !this._animate;
            }
        }
    }
    /**
     * @param {?} event
     * @return {?}
     */
    onDragMove(event) {
        if (this._dragStart) {
            event.preventDefault();
            /** @type {?} */
            let difference = (event.pageX || event.touches[0].pageX) - this._dragStart;
            if (difference < -(Number(this.width)) || difference > 0) {
                return;
            }
            this._dragEnd = difference;
        }
    }
    /**
     * @param {?} event
     * @param {?=} clearDragEnd
     * @return {?}
     */
    onDragEnd(event, clearDragEnd = false) {
        if (this._dragStart) {
            event.preventDefault();
            event.stopPropagation();
            if (this._dragEnd) {
                this.setState(this._dragEnd > -(Number(this.width) / 2));
            }
            this._dragStart = null;
            if (clearDragEnd) {
                this._dragEnd = null;
            }
            if (this._innerAnimate && !this._animate) {
                this._animate = true;
            }
        }
    }
    /**
     * @param {?=} disableAnimation
     * @return {?}
     */
    calculateWidth(disableAnimation = false) {
        if (disableAnimation && this._innerAnimate) {
            this._animate = false;
        }
        /** @type {?} */
        const initialized = (/** @type {?} */ (this._initialized));
        if (!initialized) {
            this.container$.style.width = 'auto';
            this.element$.style.width = 'auto';
        }
        this.on$.style.width = 'auto';
        this.off$.style.width = 'auto';
        setTimeout(() => {
            /** @type {?} */
            let width = this._innerWidth;
            if (this._innerWidth === 'auto') {
                width = Math.max(this.on$.offsetWidth, this.off$.offsetWidth);
            }
            this.handleWidth = this.handle$.offsetWidth;
            this.width = Number(width);
            if (!initialized) {
                this.container$.style.width = ((this.width * 2) + this.handleWidth) + 'px';
                this.element$.style.width = (this.width + this.handleWidth) + 'px';
            }
            this.ngZone.run(() => {
                this.on$.style.width = this.width + 'px';
                this.off$.style.width = this.width + 'px';
                setTimeout(() => {
                    if (disableAnimation && this._innerAnimate) {
                        this._animate = true;
                    }
                });
            });
        });
    }
    /**
     * @param {?} value
     * @return {?}
     */
    setState(value) {
        if (value !== this._innerState) {
            this._innerState = value;
            this.valueChange.emit(this._innerState);
        }
    }
    /**
     * @return {?}
     */
    get element$() {
        return this.element.nativeElement;
    }
    /**
     * @return {?}
     */
    get on$() {
        return this.onElement.nativeElement;
    }
    /**
     * @return {?}
     */
    get off$() {
        return this.offElement.nativeElement;
    }
    /**
     * @return {?}
     */
    get handle$() {
        return this.handleElement.nativeElement;
    }
    /**
     * @return {?}
     */
    get container$() {
        return this.containerElement.nativeElement;
    }
}
NgToggle.decorators = [
    { type: Component, args: [{
                selector: 'ng-toggle',
                template: `
    <span #container class="ng-toggle-container" [style.marginLeft]="marginLeft">
      <span #on class="ng-toggle-on btn btn-{{onColor}}" [class.btn-lg]="largeButton"
            [class.btn-sm]="smallButton" [class.disabled]="disabled">
          <ng-template [ngTemplateOutlet]="onLabel?.templateRef"></ng-template>
          <ng-container *ngIf="!onLabel">{{onText}}</ng-container>
      </span>
      <span #handle class="ng-toggle-handle btn" [class.btn-lg]="largeButton"
            [class.btn-sm]="smallButton" [class.disabled]="disabled"
            [class.btn-light]="!handleDark" [class.btn-dark]="handleDark">&nbsp;</span>
      <span #off class="ng-toggle-off btn btn-{{offColor}}" [class.btn-lg]="largeButton"
            [class.btn-sm]="smallButton" [class.disabled]="disabled">
        <ng-template [ngTemplateOutlet]="offLabel?.templateRef"></ng-template>
        <ng-container *ngIf="!offLabel">{{offText}}</ng-container>
      </span>
    </span>
    <ng-content></ng-content>
    `,
                preserveWhitespaces: false,
                encapsulation: ViewEncapsulation.None,
                styles: ["ng-toggle{position:relative;display:inline-block;direction:ltr;cursor:pointer;overflow:hidden;padding:0!important;text-align:left;z-index:0;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;vertical-align:middle;transition:border-color .15s ease-in-out,box-shadow .15s ease-in-out;box-sizing:content-box}ng-toggle.disabled,ng-toggle.disabled .btn{cursor:default}ng-toggle input{position:absolute;z-index:-1;visibility:hidden;width:1px;height:1px}ng-toggle .ng-toggle-container,ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{display:-ms-flexbox!important;display:flex!important}ng-toggle .ng-toggle-container{-ms-flex-align:stretch!important;align-items:stretch!important;top:0;border-radius:0;-webkit-transform:translateZ(0);transform:translateZ(0)}ng-toggle.ng-toggle-animate .ng-toggle-container{transition:margin-left .5s}ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{-ms-flex-align:center!important;align-items:center!important;text-align:center;z-index:1;border-radius:0}ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{box-sizing:border-box;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}ng-toggle .ng-toggle-handle{text-align:center;margin-top:-1px;margin-bottom:-1px;z-index:100;width:1em;padding-left:0;padding-right:0;-ms-flex-item-align:stretch!important;-ms-grid-row-align:stretch!important;align-self:stretch!important}"]
            }] }
];
/** @nocollapse */
NgToggle.ctorParameters = () => [
    { type: NgZone },
    { type: ElementRef }
];
NgToggle.propDecorators = {
    onText: [{ type: Input }],
    offText: [{ type: Input }],
    onColor: [{ type: Input }],
    offColor: [{ type: Input }],
    size: [{ type: Input }],
    disabled: [{ type: Input }, { type: HostBinding, args: ['class.disabled',] }, { type: HostBinding, args: ['class.ng-toggle-disabled',] }],
    outerAnimate: [{ type: Input, args: ['disableInitialAnimation',] }],
    value: [{ type: Input }],
    valueChange: [{ type: Output }],
    btnClass: [{ type: HostBinding, args: ['class.btn',] }],
    containerElement: [{ type: ViewChild, args: ['container',] }],
    onElement: [{ type: ViewChild, args: ['on',] }],
    offElement: [{ type: ViewChild, args: ['off',] }],
    handleElement: [{ type: ViewChild, args: ['handle',] }],
    labelElements: [{ type: ContentChildren, args: [NgToggleLabel,] }],
    largeButton: [{ type: HostBinding, args: ['class.btn-lg',] }, { type: HostBinding, args: ['class.ng-toggle-lg',] }],
    smallButton: [{ type: HostBinding, args: ['class.btn-sm',] }, { type: HostBinding, args: ['class.ng-toggle-sm',] }],
    indeterminate: [{ type: HostBinding, args: ['class.ng-toggle-indeterminate',] }],
    toggledOn: [{ type: HostBinding, args: ['class.ng-toggled-on',] }],
    toggledOff: [{ type: HostBinding, args: ['class.ng-toggled-off',] }],
    animate: [{ type: HostBinding, args: ['class.ng-toggle-animate',] }],
    handleResize: [{ type: HostListener, args: ['window:resize',] }],
    handleClick: [{ type: HostListener, args: ['click',] }],
    onTouchStart: [{ type: HostListener, args: ['touchstart', ['$event'],] }],
    onMouseDown: [{ type: HostListener, args: ['mousedown', ['$event'],] }],
    onTouchMove: [{ type: HostListener, args: ['touchmove', ['$event'],] }],
    onMouseMove: [{ type: HostListener, args: ['mousemove', ['$event'],] }],
    onTouchEnd: [{ type: HostListener, args: ['touchend', ['$event'],] }],
    onMouseUp: [{ type: HostListener, args: ['mouseup', ['$event'],] }],
    onMouseLeave: [{ type: HostListener, args: ['mouseleave', ['$event'],] }],
    onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
class NgToggleAccessor {
    /**
     * @param {?} _host
     */
    constructor(_host) {
        this._host = _host;
        this._subscription = this._host.valueChange.subscribe(value => this.onChange(value));
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        if (this._subscription) {
            this._subscription.unsubscribe();
            this._subscription = null;
        }
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onChange(_) {
        if (this._onChange) {
            this._onChange(this._host.value);
        }
    }
    /**
     * @return {?}
     */
    onTouched() {
        if (this._onTouched) {
            this._onTouched();
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    setDisabledState(isDisabled) {
        this._host.disabled = isDisabled;
    }
    /**
     * @param {?} obj
     * @return {?}
     */
    writeValue(obj) {
        if (typeof obj === 'boolean' || obj === null) {
            this._host.value = obj;
        }
    }
}
NgToggleAccessor.decorators = [
    { type: Directive, args: [{
                selector: 'ng-toggle',
                host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
                providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(() => NgToggleAccessor), multi: true }]
            },] }
];
/** @nocollapse */
NgToggleAccessor.ctorParameters = () => [
    { type: NgToggle }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
/** @type {?} */
const declarations = [
    NgToggle,
    NgToggleAccessor,
    NgToggleLabel,
];
class NgToggleModule {
}
NgToggleModule.decorators = [
    { type: NgModule, args: [{ imports: [CommonModule], exports: declarations, declarations: declarations },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */

export { NgToggle, NgToggleAccessor, NgToggleLabel, NgToggleModule };

//# sourceMappingURL=ng-toggle.js.map