import { CommonModule } from '@angular/common';
import { Directive, ElementRef, Input, TemplateRef, Component, ContentChildren, EventEmitter, HostBinding, HostListener, NgZone, Output, ViewChild, ViewEncapsulation, forwardRef, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
/**
 * The ToggleLabel directive allows you to customize the label for the "On" and "Off" states,
 * allowing for more robust and complex displays.
 * This directive must be used in conjunction with a ng-template.
 */
var NgToggleLabel = /** @class */ (function () {
    function NgToggleLabel(templateRef, elRef) {
        this.templateRef = templateRef;
        this.elRef = elRef;
    }
    Object.defineProperty(NgToggleLabel.prototype, "element", {
        get: /**
         * @return {?}
         */
        function () {
            return this.elRef;
        },
        enumerable: true,
        configurable: true
    });
    NgToggleLabel.decorators = [
        { type: Directive, args: [{ selector: 'ng-template[ngToggleLabel]' },] }
    ];
    /** @nocollapse */
    NgToggleLabel.ctorParameters = function () { return [
        { type: TemplateRef },
        { type: ElementRef }
    ]; };
    NgToggleLabel.propDecorators = {
        forLabel: [{ type: Input }]
    };
    return NgToggleLabel;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
/**
 * The Toggle directive allows for standalone or checkbox-enabled switch toggling via a UI element.
 * The toggle is styled using Bootstrap v4+ classes.
 */
var NgToggle = /** @class */ (function () {
    function NgToggle(ngZone, element) {
        this.ngZone = ngZone;
        this.element = element;
        /**
         * Display text when toggled in the "On" position
         */
        this.onText = 'On';
        /**
         * Display text when toggled in the "Off" position
         */
        this.offText = 'Off';
        /**
         * Bootstrap color scheme when toggled in the "On" position (i.e. "primary" translates to "btn-primary")
         */
        this.onColor = 'primary';
        /**
         * Bootstrap color scheme when toggled in the "Off" position (i.e. "primary" translates to "btn-primary")
         */
        this.offColor = 'secondary';
        /**
         * Button size to display the toggle
         */
        this.size = '';
        /**
         * Whether the toggle is disabled or not
         */
        this.disabled = false;
        /**
         * An event fired when the user causes a change.
         * The payload of the event is the currently selected value.
         */
        this.valueChange = new EventEmitter();
        this.btnClass = true;
        this.width = 0;
        this.handleWidth = 0;
        this._animate = true;
        this._innerAnimate = true;
        this._innerState = false;
        this._innerWidth = 'auto';
        this._disableInitialAnimation = false;
        this._dragStart = null;
        this._dragEnd = null;
        this._initialized = false;
        this._hidden = false;
    }
    Object.defineProperty(NgToggle.prototype, "outerAnimate", {
        /**
         * Optional. Enable/Disable the initial transition animation. May prevent unwanted animation display.
         */
        set: /**
         * Optional. Enable/Disable the initial transition animation. May prevent unwanted animation display.
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this._disableInitialAnimation = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () {
            return this._innerState;
        },
        /**
         * @param {boolean} value
         */
        set: /**
         * @param {?} value
         * @return {?}
         */
        function (value) {
            this.setState(value);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NgToggle.prototype.ngAfterViewInit = /**
     * @return {?}
     */
    function () {
        var _this = this;
        this.calculateWidth();
        this._initialized = true;
        this.ngZone.run(function () { return setTimeout(function () { return _this._disableInitialAnimation = false; }); });
    };
    /**
     * @return {?}
     */
    NgToggle.prototype.ngAfterViewChecked = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var hidden = this.element.nativeElement.offsetParent === null;
        if ((!this._initialized || this.width === 0) && this._hidden !== hidden) {
            this._initialized = false;
            this.calculateWidth();
            this._initialized = true;
        }
        this._hidden = hidden;
    };
    /**
     * @return {?}
     */
    NgToggle.prototype.ngAfterContentInit = /**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var onElement = this.labelElements.find(function (item) { return item.forLabel.toLowerCase() === 'on'; });
        /** @type {?} */
        var offElement = this.labelElements.find(function (item) { return item.forLabel.toLowerCase() === 'off'; });
        if (onElement) {
            this.onLabel = onElement;
        }
        if (offElement) {
            this.offLabel = offElement;
        }
        if (onElement || offElement) {
            this.calculateWidth(true);
        }
    };
    /**
     * @param {?} changes
     * @return {?}
     */
    NgToggle.prototype.ngOnChanges = /**
     * @param {?} changes
     * @return {?}
     */
    function (changes) {
        if ('onText' in changes || 'offText' in changes || 'size' in changes) {
            this._initialized = false;
            this.calculateWidth(this._initialized);
            this._initialized = true;
        }
    };
    Object.defineProperty(NgToggle.prototype, "largeButton", {
        get: /**
         * @return {?}
         */
        function () {
            return this.size === 'lg';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "smallButton", {
        get: /**
         * @return {?}
         */
        function () {
            return this.size === 'sm';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "handleDark", {
        get: /**
         * @return {?}
         */
        function () {
            return (this.value && this.onColor === 'light') || (!this.value && this.offColor === 'light');
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "indeterminate", {
        get: /**
         * @return {?}
         */
        function () {
            return this._innerState === null || typeof this._innerState === 'undefined';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "toggledOn", {
        get: /**
         * @return {?}
         */
        function () {
            return this.innerState === true;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "toggledOff", {
        get: /**
         * @return {?}
         */
        function () {
            return this.innerState === false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "innerState", {
        get: /**
         * @return {?}
         */
        function () {
            return this._innerState;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "animate", {
        get: /**
         * @return {?}
         */
        function () {
            return this._animate && (!this._disableInitialAnimation || this._initialized && !this._disableInitialAnimation);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "marginLeft", {
        get: /**
         * @return {?}
         */
        function () {
            /** @type {?} */
            var margin = 0;
            if (!this._initialized) ;
            else if (this.indeterminate || this._innerState === null || typeof this._innerState === 'undefined') {
                margin = -(this.width / 2);
            }
            else if (this._dragEnd) {
                margin = this._dragEnd;
            }
            else if (!this._innerState) {
                margin = -this.width;
            }
            return margin + 'px';
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    NgToggle.prototype.handleResize = /**
     * @return {?}
     */
    function () {
        this._initialized = false;
        this.calculateWidth(true);
        this._initialized = true;
    };
    /**
     * @return {?}
     */
    NgToggle.prototype.handleClick = /**
     * @return {?}
     */
    function () {
        if (!this.disabled && !this._dragEnd) {
            this.setState(!this._innerState);
        }
        else if (this._dragEnd) {
            this._dragEnd = null;
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onTouchStart = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragStart(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onMouseDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragStart(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onTouchMove = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragMove(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onMouseMove = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragMove(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onTouchEnd = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragEnd(event, true);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onMouseUp = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragEnd(event);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onMouseLeave = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        this.onDragEnd(event, true);
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onKeyDown = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (!event.key || this.disabled) {
            return;
        }
        switch (event.key) {
            case 'Left':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(false);
                break;
            case 'Right':
                event.preventDefault();
                event.stopImmediatePropagation();
                this.setState(true);
                break;
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onDragStart = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (event.target === this.handle$) {
            if (this._dragStart || this.disabled) {
                return;
            }
            event.preventDefault();
            event.stopPropagation();
            this._dragStart = (event.pageX || event.touches[0].pageX) - parseInt(this.container$.style.marginLeft, 10);
            if (this._animate) {
                this._animate = !this._animate;
            }
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    NgToggle.prototype.onDragMove = /**
     * @param {?} event
     * @return {?}
     */
    function (event) {
        if (this._dragStart) {
            event.preventDefault();
            /** @type {?} */
            var difference = (event.pageX || event.touches[0].pageX) - this._dragStart;
            if (difference < -(Number(this.width)) || difference > 0) {
                return;
            }
            this._dragEnd = difference;
        }
    };
    /**
     * @param {?} event
     * @param {?=} clearDragEnd
     * @return {?}
     */
    NgToggle.prototype.onDragEnd = /**
     * @param {?} event
     * @param {?=} clearDragEnd
     * @return {?}
     */
    function (event, clearDragEnd) {
        if (clearDragEnd === void 0) { clearDragEnd = false; }
        if (this._dragStart) {
            event.preventDefault();
            event.stopPropagation();
            if (this._dragEnd) {
                this.setState(this._dragEnd > -(Number(this.width) / 2));
            }
            this._dragStart = null;
            if (clearDragEnd) {
                this._dragEnd = null;
            }
            if (this._innerAnimate && !this._animate) {
                this._animate = true;
            }
        }
    };
    /**
     * @param {?=} disableAnimation
     * @return {?}
     */
    NgToggle.prototype.calculateWidth = /**
     * @param {?=} disableAnimation
     * @return {?}
     */
    function (disableAnimation) {
        var _this = this;
        if (disableAnimation === void 0) { disableAnimation = false; }
        if (disableAnimation && this._innerAnimate) {
            this._animate = false;
        }
        /** @type {?} */
        var initialized = (/** @type {?} */ (this._initialized));
        if (!initialized) {
            this.container$.style.width = 'auto';
            this.element$.style.width = 'auto';
        }
        this.on$.style.width = 'auto';
        this.off$.style.width = 'auto';
        setTimeout(function () {
            /** @type {?} */
            var width = _this._innerWidth;
            if (_this._innerWidth === 'auto') {
                width = Math.max(_this.on$.offsetWidth, _this.off$.offsetWidth);
            }
            _this.handleWidth = _this.handle$.offsetWidth;
            _this.width = Number(width);
            if (!initialized) {
                _this.container$.style.width = ((_this.width * 2) + _this.handleWidth) + 'px';
                _this.element$.style.width = (_this.width + _this.handleWidth) + 'px';
            }
            _this.ngZone.run(function () {
                _this.on$.style.width = _this.width + 'px';
                _this.off$.style.width = _this.width + 'px';
                setTimeout(function () {
                    if (disableAnimation && _this._innerAnimate) {
                        _this._animate = true;
                    }
                });
            });
        });
    };
    /**
     * @param {?} value
     * @return {?}
     */
    NgToggle.prototype.setState = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value !== this._innerState) {
            this._innerState = value;
            this.valueChange.emit(this._innerState);
        }
    };
    Object.defineProperty(NgToggle.prototype, "element$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.element.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "on$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.onElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "off$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.offElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "handle$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.handleElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(NgToggle.prototype, "container$", {
        get: /**
         * @return {?}
         */
        function () {
            return this.containerElement.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    NgToggle.decorators = [
        { type: Component, args: [{
                    selector: 'ng-toggle',
                    template: "\n    <span #container class=\"ng-toggle-container\" [style.marginLeft]=\"marginLeft\">\n      <span #on class=\"ng-toggle-on btn btn-{{onColor}}\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\">\n          <ng-template [ngTemplateOutlet]=\"onLabel?.templateRef\"></ng-template>\n          <ng-container *ngIf=\"!onLabel\">{{onText}}</ng-container>\n      </span>\n      <span #handle class=\"ng-toggle-handle btn\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\"\n            [class.btn-light]=\"!handleDark\" [class.btn-dark]=\"handleDark\">&nbsp;</span>\n      <span #off class=\"ng-toggle-off btn btn-{{offColor}}\" [class.btn-lg]=\"largeButton\"\n            [class.btn-sm]=\"smallButton\" [class.disabled]=\"disabled\">\n        <ng-template [ngTemplateOutlet]=\"offLabel?.templateRef\"></ng-template>\n        <ng-container *ngIf=\"!offLabel\">{{offText}}</ng-container>\n      </span>\n    </span>\n    <ng-content></ng-content>\n    ",
                    preserveWhitespaces: false,
                    encapsulation: ViewEncapsulation.None,
                    styles: ["ng-toggle{position:relative;display:inline-block;direction:ltr;cursor:pointer;overflow:hidden;padding:0!important;text-align:left;z-index:0;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none;vertical-align:middle;transition:border-color .15s ease-in-out,box-shadow .15s ease-in-out;box-sizing:content-box}ng-toggle.disabled,ng-toggle.disabled .btn{cursor:default}ng-toggle input{position:absolute;z-index:-1;visibility:hidden;width:1px;height:1px}ng-toggle .ng-toggle-container,ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{display:-ms-flexbox!important;display:flex!important}ng-toggle .ng-toggle-container{-ms-flex-align:stretch!important;align-items:stretch!important;top:0;border-radius:0;-webkit-transform:translateZ(0);transform:translateZ(0)}ng-toggle.ng-toggle-animate .ng-toggle-container{transition:margin-left .5s}ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{-ms-flex-align:center!important;align-items:center!important;text-align:center;z-index:1;border-radius:0}ng-toggle .ng-toggle-handle,ng-toggle .ng-toggle-off,ng-toggle .ng-toggle-on{box-sizing:border-box;cursor:pointer;-webkit-user-select:none;-moz-user-select:none;-ms-user-select:none;user-select:none}ng-toggle .ng-toggle-handle{text-align:center;margin-top:-1px;margin-bottom:-1px;z-index:100;width:1em;padding-left:0;padding-right:0;-ms-flex-item-align:stretch!important;-ms-grid-row-align:stretch!important;align-self:stretch!important}"]
                }] }
    ];
    /** @nocollapse */
    NgToggle.ctorParameters = function () { return [
        { type: NgZone },
        { type: ElementRef }
    ]; };
    NgToggle.propDecorators = {
        onText: [{ type: Input }],
        offText: [{ type: Input }],
        onColor: [{ type: Input }],
        offColor: [{ type: Input }],
        size: [{ type: Input }],
        disabled: [{ type: Input }, { type: HostBinding, args: ['class.disabled',] }, { type: HostBinding, args: ['class.ng-toggle-disabled',] }],
        outerAnimate: [{ type: Input, args: ['disableInitialAnimation',] }],
        value: [{ type: Input }],
        valueChange: [{ type: Output }],
        btnClass: [{ type: HostBinding, args: ['class.btn',] }],
        containerElement: [{ type: ViewChild, args: ['container',] }],
        onElement: [{ type: ViewChild, args: ['on',] }],
        offElement: [{ type: ViewChild, args: ['off',] }],
        handleElement: [{ type: ViewChild, args: ['handle',] }],
        labelElements: [{ type: ContentChildren, args: [NgToggleLabel,] }],
        largeButton: [{ type: HostBinding, args: ['class.btn-lg',] }, { type: HostBinding, args: ['class.ng-toggle-lg',] }],
        smallButton: [{ type: HostBinding, args: ['class.btn-sm',] }, { type: HostBinding, args: ['class.ng-toggle-sm',] }],
        indeterminate: [{ type: HostBinding, args: ['class.ng-toggle-indeterminate',] }],
        toggledOn: [{ type: HostBinding, args: ['class.ng-toggled-on',] }],
        toggledOff: [{ type: HostBinding, args: ['class.ng-toggled-off',] }],
        animate: [{ type: HostBinding, args: ['class.ng-toggle-animate',] }],
        handleResize: [{ type: HostListener, args: ['window:resize',] }],
        handleClick: [{ type: HostListener, args: ['click',] }],
        onTouchStart: [{ type: HostListener, args: ['touchstart', ['$event'],] }],
        onMouseDown: [{ type: HostListener, args: ['mousedown', ['$event'],] }],
        onTouchMove: [{ type: HostListener, args: ['touchmove', ['$event'],] }],
        onMouseMove: [{ type: HostListener, args: ['mousemove', ['$event'],] }],
        onTouchEnd: [{ type: HostListener, args: ['touchend', ['$event'],] }],
        onMouseUp: [{ type: HostListener, args: ['mouseup', ['$event'],] }],
        onMouseLeave: [{ type: HostListener, args: ['mouseleave', ['$event'],] }],
        onKeyDown: [{ type: HostListener, args: ['keydown', ['$event'],] }]
    };
    return NgToggle;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
var NgToggleAccessor = /** @class */ (function () {
    function NgToggleAccessor(_host) {
        var _this = this;
        this._host = _host;
        this._subscription = this._host.valueChange.subscribe(function (value) { return _this.onChange(value); });
    }
    /**
     * @return {?}
     */
    NgToggleAccessor.prototype.ngOnDestroy = /**
     * @return {?}
     */
    function () {
        if (this._subscription) {
            this._subscription.unsubscribe();
            this._subscription = null;
        }
    };
    /**
     * @param {?} _
     * @return {?}
     */
    NgToggleAccessor.prototype.onChange = /**
     * @param {?} _
     * @return {?}
     */
    function (_) {
        if (this._onChange) {
            this._onChange(this._host.value);
        }
    };
    /**
     * @return {?}
     */
    NgToggleAccessor.prototype.onTouched = /**
     * @return {?}
     */
    function () {
        if (this._onTouched) {
            this._onTouched();
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NgToggleAccessor.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NgToggleAccessor.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouched = fn;
    };
    /**
     * @param {?} isDisabled
     * @return {?}
     */
    NgToggleAccessor.prototype.setDisabledState = /**
     * @param {?} isDisabled
     * @return {?}
     */
    function (isDisabled) {
        this._host.disabled = isDisabled;
    };
    /**
     * @param {?} obj
     * @return {?}
     */
    NgToggleAccessor.prototype.writeValue = /**
     * @param {?} obj
     * @return {?}
     */
    function (obj) {
        if (typeof obj === 'boolean' || obj === null) {
            this._host.value = obj;
        }
    };
    NgToggleAccessor.decorators = [
        { type: Directive, args: [{
                    selector: 'ng-toggle',
                    host: { '(change)': 'onChange($event)', '(touch)': 'onTouched()' },
                    providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: forwardRef(function () { return NgToggleAccessor; }), multi: true }]
                },] }
    ];
    /** @nocollapse */
    NgToggleAccessor.ctorParameters = function () { return [
        { type: NgToggle }
    ]; };
    return NgToggleAccessor;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */
/** @type {?} */
var declarations = [
    NgToggle,
    NgToggleAccessor,
    NgToggleLabel,
];
var NgToggleModule = /** @class */ (function () {
    function NgToggleModule() {
    }
    NgToggleModule.decorators = [
        { type: NgModule, args: [{ imports: [CommonModule], exports: declarations, declarations: declarations },] }
    ];
    return NgToggleModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,uselessCode} checked by tsc
 */

export { NgToggle, NgToggleAccessor, NgToggleLabel, NgToggleModule };

//# sourceMappingURL=ng-toggle.js.map