import { AfterContentInit, AfterViewChecked, AfterViewInit, ElementRef, EventEmitter, NgZone, OnChanges, QueryList, SimpleChanges } from '@angular/core';
import { NgToggleLabel } from './toggle-label';
/**
 * The Toggle directive allows for standalone or checkbox-enabled switch toggling via a UI element.
 * The toggle is styled using Bootstrap v4+ classes.
 */
export declare class NgToggle implements AfterViewInit, AfterContentInit, AfterViewChecked, OnChanges {
    private ngZone;
    private element;
    /**
     * Display text when toggled in the "On" position
     * @type {string}
     */
    onText: string;
    /**
     * Display text when toggled in the "Off" position
     * @type {string}
     */
    offText: string;
    /**
     * Bootstrap color scheme when toggled in the "On" position (i.e. "primary" translates to "btn-primary")
     * @type {string}
     */
    onColor: string;
    /**
     * Bootstrap color scheme when toggled in the "Off" position (i.e. "primary" translates to "btn-primary")
     * @type {string}
     */
    offColor: string;
    /**
     * Button size to display the toggle
     * @type {string}
     */
    size: 'sm' | 'lg' | '';
    /**
     * Whether the toggle is disabled or not
     * @type {boolean}
     */
    disabled: boolean;
    /**
     * Optional. Enable/Disable the initial transition animation. May prevent unwanted animation display.
     */
    outerAnimate: boolean;
    /**
     * @param {boolean} value
     */
    value: boolean;
    /**
     * An event fired when the user causes a change.
     * The payload of the event is the currently selected value.
     * @type {EventEmitter<boolean>}
     */
    valueChange: EventEmitter<boolean>;
    btnClass: boolean;
    width: number;
    handleWidth: number;
    containerElement: ElementRef;
    onElement: ElementRef;
    offElement: ElementRef;
    handleElement: ElementRef;
    labelElements: QueryList<NgToggleLabel>;
    onLabel: NgToggleLabel;
    offLabel: NgToggleLabel;
    private _animate;
    private _innerAnimate;
    private _innerState;
    private _innerWidth;
    private _disableInitialAnimation;
    private _dragStart;
    private _dragEnd;
    private _initialized;
    private _hidden;
    constructor(ngZone: NgZone, element: ElementRef);
    ngAfterViewInit(): void;
    ngAfterViewChecked(): void;
    ngAfterContentInit(): void;
    ngOnChanges(changes: SimpleChanges): void;
    readonly largeButton: boolean;
    readonly smallButton: boolean;
    readonly handleDark: boolean;
    readonly indeterminate: boolean;
    readonly toggledOn: boolean;
    readonly toggledOff: boolean;
    readonly innerState: boolean;
    readonly animate: boolean;
    readonly marginLeft: string;
    handleResize(): void;
    handleClick(): void;
    onTouchStart(event: any): void;
    onMouseDown(event: any): void;
    onTouchMove(event: any): void;
    onMouseMove(event: any): void;
    onTouchEnd(event: any): void;
    onMouseUp(event: any): void;
    onMouseLeave(event: any): void;
    onKeyDown(event: KeyboardEvent): void;
    private onDragStart;
    private onDragMove;
    private onDragEnd;
    private calculateWidth;
    private setState;
    private readonly element$;
    private readonly on$;
    private readonly off$;
    private readonly handle$;
    private readonly container$;
}
